#-------------------------------------------------------------------------------
# Name:        Guess the Number
# Purpose:     Number guessing game
#
# Author:      PMP
#
# Created:     12/01/2013
# Copyright:   (c) PMP 2013
# Licence:     n/a
#-------------------------------------------------------------------------------

# This is a Guess the Number game from 'Invent Your Own Computer Game With Python' adapted by PMP.
# This version (07) puts the main game loop into  def main() .


# Variables in a user-defined function (def) are local by default.
# If a variable in a def is declared as global, references to it in defs executed later usually treat it as global without specific redeclaration.
# If a def uses a statement like x = x + 1 and x is intended to be a global variable, it must be declared as global in the def, even if it has been declared already in an earlier executed def.


import random as r  # This imports a module to add randomizing functions to the basic functions available in Python.
import math as m    # This imports a module to add mathematical functions to the basic functions available in Python.


def main():                           # This is the game main function.
    displayTitle()
    playerN = inputName()
    minNum = setNumberLimitLow()
    maxNum = setNumberLimitHigh(minNum)
    maxGuess = setMaxGuesses(minNum, maxNum)

    while True:                       # This 'while' loop runs as long as the player asks to continue playing after each game.
        numberC = startGame(playerN, minNum, maxNum, maxGuess)
        guessTaken = 0
        while guessTaken < maxGuess:  # This 'while' loop runs as long as the number of guesses taken is less than the maximum number of guesses.
            askForGuess(maxGuess, guessTaken)
            mGuess = makeGuess()
            guessTaken = guessTaken + 1
            guessedN = checkGuess(minNum, maxNum, numberC, mGuess)
            if guessedN == True:
                break
        displayResult(playerN, numberC, mGuess, guessTaken)
        replay = newGameRequest(playerN)
        if replay == False:
            break


def displayTitle():
    print('\n' + '*' * 26 + '\n*' + ' ' * 24 + '*\n*  Number Guessing Game  *\n*' + ' ' * 24 + '*\n' + '*' * 26 + '\n\n')
    # At the start of the game, this gives a title surrounded by asterisks.
    # Note that  '*' * 26  means a string of 26 asterisks,  ' ' * 24  means a string of 24 spaces,  and  \n  means 'start a new line'.


def inputName():
    print('Hello! What is your name?')
    playerName = input()  # This allows the player to enter his/her name.
    return playerName


def setNumberLimitLow():
    print('\nWhat is the lowest number the computer may choose?\nIt must be an integer between 1 and 1,000,000.')
    while True:
        while True:                         # This 'while' loop checks whether the player's entry is valid (i.e. a number, not a letter or word).
            minNumber = input()             # This allows the player to enter a number.
            try:                            # The 'try' statement allows the program to check whether the player's entry is valid.
                minNumber = int(minNumber)  # This tries to treat the player's entry as an integer number.
                break                       # If the player's entry is something which can be converted to an integer, then 'break' out of this 'while' loop.
            except ValueError:              # If the player's entry cannot be converted to an integer, then print an error message.
                print('You must enter integers, not fractions or letters - try again.')
        minNumber = int(minNumber)
        if minNumber < 1 or minNumber > 1000000:
            print('This number is outside the acceptable range - try again.')
        else:
            break
    return minNumber


def setNumberLimitHigh(minNumber):
    print('\nWhat is the highest number the computer may choose?\nIt must be an integer between 1 and 1,000,000 and at least 9 greater than the lowest number.')
    while True:
        while True:                         # This 'while' loop checks whether the player's entry is valid (i.e. a number, not a letter or word).
            maxNumber = input()             # This allows the player to enter a number.
            try:                            # The 'try' statement allows the program to check whether the player's entry is valid.
                maxNumber = int(maxNumber)  # This tries to treat the player's entry as an integer number.
                break                       # If the player's entry is something which can be converted to an integer, then 'break' out of this 'while' loop.
            except ValueError:              # If the player's entry cannot be converted to an integer, then print an error message.
                print('You must enter integers, not fractions or letters - try again.')
        maxNumber = int(maxNumber)
        if maxNumber < 1 or maxNumber > 1000000 or maxNumber < minNumber + 9:
            print('This number is outside the acceptable range - try again.')
        else:
            break
    return maxNumber


def setMaxGuesses(minNumber, maxNumber):
    maxGuesses = int(m.log(maxNumber + 1 - minNumber, 2) + 1)
    return maxGuesses


def startGame(playerName, minNumber, maxNumber, maxGuesses):
    numberChosen = r.randint(minNumber, maxNumber)  # This sets the variable 'number' to an integer between 'minNumber' and 'maxNumber' inclusive.
    print('\nWell, ' + playerName + ', I\'m thinking of a number between ' + str(minNumber) + ' and ' + str(maxNumber) + '.')
    print('You have ' + str(maxGuesses) + ' guesses in which to work it out.')
    # \' allows an apostrophe to be used within a string.
    # str() converts a numeric variable to a string variable (so that it can be displayed correctly with 'print').
    return numberChosen


def askForGuess(maxGuesses, guessesTaken):
    print()                            # This prints a blank line.
    if maxGuesses - guessesTaken > 1:  # This 'if' statement gives an output if the player has more than one guess left.
        print('You have ' + str(maxGuesses - guessesTaken) + ' guesses left.')
    else:
        print('You have 1 guess left.')
    print('What do you think my number is?')


def makeGuess():
    while True:                 # This 'while' loop checks whether the player's guess is valid (i.e. a number, not a letter or word).
        guess = input()         # This allows the player to enter a guess.
        try:                    # The 'try' statement allows the program to check whether the player's entry is valid.
            guess = int(guess)  # This tries to treat the player's entry as an integer number.
            break               # If the player's entry is something which can be converted to an integer, then 'break' out of this 'while' loop.
        except ValueError:      # If the player's entry cannot be converted to an integer, then print an error message.
            print('You must enter numbers, not letters - try again.')
    guess = int(guess)          # This converts the player's (string) entry to an integer.
    return guess


def checkGuess(minNumber, maxNumber, numberChosen, guess):
    guessed = False
    if guess < numberChosen:     # This 'if' statement gives an output if the player's guess is less than the computer's chosen number.
        print('Your guess is too low.')
        if guess < minNumber:    # This 'if' statement gives an output if the player's guess is less than 'minNumber'.
            print('You\'ve wasted this guess, as your number is below the minimum possible choice for the computer.')

    elif guess > numberChosen:   # This 'elif' statement gives an output if the player's guess is greater than the computer's chosen number.
        print('Your guess is too high.')
        if guess > maxNumber:    # This 'if' statement gives an output if the player's guess is greater than 'maxNumber'.
            print('You\'ve wasted this guess, as your number is above the maximum possible choice for the computer.')

    elif guess == numberChosen:  # This 'elif' statement gives an output if the player's guess is equal to the computer's chosen number.
        guessed = True
    return guessed


def displayResult(playerName, numberChosen, guess, guessesTaken):
    if guess == numberChosen:  # This 'if' statement gives an output depending on whether the player got the computer's number right or not.
        print('\nWell done, ' + playerName + '! You guessed my number in ', end='')  # , end=''  makes the next 'print' function continue on the same line.
        if guessesTaken > 1:   # This 'if' statement gives an output depending on how many guesses the player took.
            print(str(guessesTaken) + ' guesses!')
        else:
            print('one guess!')
    else:
        print('\nBad luck. The number I was thinking of was ' + str(numberChosen) + '.')


def newGameRequest(playerName):
    replay = True
    print('\nDo you want another game, ' + playerName + '?')
    newGameYesNo = str.lower(input())                  # 'str.lower' converts a string to lower case.
    if newGameYesNo != 'yes' and newGameYesNo != 'y':  # This 'if' statement ends the program if the player enters anything other than 'yes' or 'y'.
        print('\nThank you for playing \'Guess The Number\', ' + playerName + ".")
        replay = False
    return replay



if __name__ == '__main__':              # This runs  main()  if this module has not been imported into another module.
    main()
