#-------------------------------------------------------------------------------
# Name:        Guess the Number
# Purpose:     Number guessing game
#
# Author:      PMP
#
# Created:     19/01/2013
# Copyright:   (c) PMP 2013
# Licence:     n/a
#-------------------------------------------------------------------------------

# This is a Guess the Number game from 'Invent Your Own Computer Game With Python' adapted by PMP.
# This version (08) puts the game in a window using PyGame and PygCurse.


# Variables in a user-defined function (def) are local by default.
# If a variable in a def is declared as global, references to it in defs executed later usually treat it as global without specific redeclaration.
# If a def uses a statement like x = x + 1 and x is intended to be a global variable, it must be declared as global in the def, even if it has been declared already in an earlier executed def.

from sys import exit     # This imports just the exit function from Sys.
from pygame import quit  # This imports just the quit function from PyGame.
import pygcurse as pc    # This imports the PygCurse text console add-on for PyGame.
import random as r       # This imports randomizing functions.
import math as m         # This imports mathematical functions.

win = pc.PygcurseWindow(27, 30, 'Guess the Number')  # Creates a window 27 characters wide x 30 lines high.
print = win.pygprint                                 # Makes the PygCurse pygprint function replace standard print().
input = win.input                                    # Makes the PygCurse input function replace standard input().
win.setscreencolors('white', 'black', clear=True)    # Sets the colours for the window and clears it (necessary to allow proper scrolling).


def main():                           # This is the game main function.
    displayTitle()
    playerN = inputName()
    minNum = setNumberLimitLow()
    maxNum = setNumberLimitHigh(minNum)
    maxGuess = setMaxGuesses(minNum, maxNum)

    while True:                       # This 'while' loop runs as long as the player asks to continue playing after each game.
        numberC = startGame(playerN, minNum, maxNum, maxGuess)
        guessTaken = 0
        while guessTaken < maxGuess:  # This 'while' loop runs as long as the number of guesses taken is less than the maximum number of guesses.
            askForGuess(maxGuess, guessTaken)
            mGuess = makeGuess()
            guessTaken = guessTaken + 1
            guessedN = checkGuess(minNum, maxNum, numberC, mGuess)
            if guessedN == True:
                break
        displayResult(playerN, numberC, mGuess, guessTaken)
        replay = newGameRequest(playerN)
        if replay == False:
            break

    quit()  # This closes the PyGame / PygCurse window.
    exit()  # This exits the program.


def displayTitle():
    win.colors = ('blue', 'yellow')
    print('\n' + '*' * 26 + '\n*' + ' ' * 24 + '*\n*  Number Guessing Game  *\n*' + ' ' * 24 + '*\n' + '*' * 26 + '\n\n')
    # At the start of the game, this gives a title surrounded by asterisks.
    # Note that  '*' * 26  means a string of 26 asterisks,  ' ' * 24  means a string of 24 spaces,  and  \n  means 'start a new line'.


def inputName():
    win.colors = ('white', 'black')
    print('Hello! What is your name?')
    win.fgcolor = 'aqua'
    playerName = input()  # This allows the player to enter his/her name.
    return playerName


def setNumberLimitLow():
    win.colors = ('white', 'black')
    print('\nWhat is the lowest number\nthe computer may choose?\nIt must be an integer\nbetween 1 and 1,000,000.')
    while True:
        while True:                         # This 'while' loop checks whether the player's entry is valid (i.e. a number, not a letter or word).
            win.fgcolor = 'aqua'
            minNumber = input()             # This allows the player to enter a number.
            try:                            # The 'try' statement allows the program to check whether the player's entry is valid.
                minNumber = int(minNumber)  # This tries to treat the player's entry as an integer number.
                break                       # If the player's entry is something which can be converted to an integer, then 'break' out of this 'while' loop.
            except ValueError:              # If the player's entry cannot be converted to an integer, then print an error message.
                win.fgcolor = 'red'
                print('You must enter integers,\nnot fractions or letters -\ntry again.')
        minNumber = int(minNumber)
        if minNumber < 1 or minNumber > 1000000:
            win.fgcolor = 'red'
            print('This number is outside\nthe acceptable range -\ntry again.')
        else:
            break
    return minNumber


def setNumberLimitHigh(minNumber):
    win.colors = ('white', 'black')
    print('\nWhat is the highest number\nthe computer may choose?\nIt must be an integer\nbetween 1 and 1,000,000\nand at least 9 greater\nthan the lowest number.')
    while True:
        while True:                         # This 'while' loop checks whether the player's entry is valid (i.e. a number, not a letter or word).
            win.fgcolor = 'aqua'
            maxNumber = input()             # This allows the player to enter a number.
            try:                            # The 'try' statement allows the program to check whether the player's entry is valid.
                maxNumber = int(maxNumber)  # This tries to treat the player's entry as an integer number.
                break                       # If the player's entry is something which can be converted to an integer, then 'break' out of this 'while' loop.
            except ValueError:              # If the player's entry cannot be converted to an integer, then print an error message.
                win.fgcolor = 'red'
                print('You must enter integers,\nnot fractions or letters -\ntry again.')
        maxNumber = int(maxNumber)
        if maxNumber < 1 or maxNumber > 1000000 or maxNumber < minNumber + 9:
            win.fgcolor = 'red'
            print('This number is outside\nthe acceptable range -\ntry again.')
        else:
            break
    return maxNumber


def setMaxGuesses(minNumber, maxNumber):
    maxGuesses = int(m.log(maxNumber + 1 - minNumber, 2) + 1)
    return maxGuesses


def startGame(playerName, minNumber, maxNumber, maxGuesses):
    win.colors = ('lime', 'black')
    numberChosen = r.randint(minNumber, maxNumber)  # This sets the variable 'number' to an integer between 'minNumber' and 'maxNumber' inclusive.
    print('\nWell, ' + playerName + ',\n  I\'m thinking of a number\n  between ' + str(minNumber) + ' and ' + str(maxNumber) + '.')
    print('You have ' + str(maxGuesses) + ' guesses\n  in which to work it out.')
    # \' allows an apostrophe to be used within a string.
    # str() converts a numeric variable to a string variable (so that it can be displayed correctly with 'print').
    return numberChosen


def askForGuess(maxGuesses, guessesTaken):
    win.colors = ('lime', 'black')
    print()                            # This prints a blank line.
    if maxGuesses - guessesTaken > 1:  # This 'if' statement gives an output if the player has more than one guess left.
        print('You have ' + str(maxGuesses - guessesTaken) + ' guesses left.')
    else:
        print('You have 1 guess left.')
    print('What do you think\n  my number is?')


def makeGuess():
    win.colors = ('aqua', 'black')
    while True:                 # This 'while' loop checks whether the player's guess is valid (i.e. a number, not a letter or word).
        guess = input()         # This allows the player to enter a guess.
        try:                    # The 'try' statement allows the program to check whether the player's entry is valid.
            guess = int(guess)  # This tries to treat the player's entry as an integer number.
            break               # If the player's entry is something which can be converted to an integer, then 'break' out of this 'while' loop.
        except ValueError:      # If the player's entry cannot be converted to an integer, then print an error message.
            win.fgcolor = 'red'
            print('You must enter numbers,\nnot letters -\ntry again.')
    guess = int(guess)          # This converts the player's (string) entry to an integer.
    return guess


def checkGuess(minNumber, maxNumber, numberChosen, guess):
    win.colors = ('yellow', 'black')
    guessed = False
    if guess < numberChosen:     # This 'if' statement gives an output if the player's guess is less than the computer's chosen number.
        print('Your guess is too low.')
        if guess < minNumber:    # This 'if' statement gives an output if the player's guess is less than 'minNumber'.
            win.fgcolor = 'red'
            print('You\'ve wasted this guess,\nas your number is below\nthe minimum possible\nchoice for the computer.')

    elif guess > numberChosen:   # This 'elif' statement gives an output if the player's guess is greater than the computer's chosen number.
        print('Your guess is too high.')
        if guess > maxNumber:    # This 'if' statement gives an output if the player's guess is greater than 'maxNumber'.
            win.fgcolor = 'red'
            print('You\'ve wasted this guess,\nas your number is above\nthe maximum possible\nchoice for the computer.')

    elif guess == numberChosen:  # This 'elif' statement gives an output if the player's guess is equal to the computer's chosen number.
        guessed = True
    return guessed


def displayResult(playerName, numberChosen, guess, guessesTaken):
    win.colors = ('lime', 'black')
    if guess == numberChosen:  # This 'if' statement gives an output depending on whether the player got the computer's number right or not.
        print('\nWell done, ' + playerName + '!\nYou guessed my number\nin ', end='')  # , end=''  makes the next 'print' function continue on the same line.
        if guessesTaken > 1:   # This 'if' statement gives an output depending on how many guesses the player took.
            print(str(guessesTaken) + ' guesses!')
        else:
            print('one guess!')
    else:
        win.fgcolor = 'red'
        print('\nBad luck.\nThe number I was thinking\nof was ' + str(numberChosen) + '.')


def newGameRequest(playerName):
    win.colors = ('lime', 'black')
    replay = True
    print('\nDo you want another game,\n' + playerName + '?')
    win.fgcolor = 'aqua'
    newGameYesNo = str.lower(input())                  # 'str.lower' converts a string to lower case.
    win.fgcolor = 'lime'
    if newGameYesNo != 'yes' and newGameYesNo != 'y':  # This 'if' statement ends the program if the player enters anything other than 'yes' or 'y'.
        print('\nThank you for playing\n\'Guess The Number\',\n' + playerName + ".")
        win.fgcolor = 'yellow'
        print('\nPress a key\n  to close the window.')
        pc.waitforkeypress()
        replay = False
    return replay



if __name__ == '__main__':  # This runs  main()  if this module has not been imported into another module.
    main()
