#-------------------------------------------------------------------------------
# Name:        Guess the Number
# Purpose:     Number guessing game
#
# Author:      PMP
#
# Created:     20/01/2013
# Copyright:   (c) PMP 2013
# Licence:     n/a
#-------------------------------------------------------------------------------

# This is a Guess the Number game from 'Invent Your Own Computer Game With Python' adapted by PMP.
# This version (09) converts the game to a GUI base using Tkinter.


import tkinter as tk             # Adds a module to Python to allow GUI development.
#import tkinter.ttk as ttk        # Adds new-style widgets to tkinter (not used in this program).
import tkinter.messagebox as tm  # Adds a module to Python to allow message boxes to be used.
import random as r               # Adds a module to Python to allow random numbers to be generated.
import math as m                 # Adds a module to Python containing various mathematical functions.


class App:

    def __init__(self, master):

        self.fontMain = ('MS Sans', 12)     # Sets the main font to use for widgets.
        self.fontSpace = ('MS Sans', 4)     # Sets the font for labels used as spacers in a frame.

        master.title('Guess the Number')    # Puts a title on the program window.
        master.geometry('360x200+400+200')  # Sets the window size (360x200) and initial screen position (400 across, 200 down from top left).

        self.frameSetupGame = tk.Frame(master, width=360, height=90, padx=4, pady=8, borderwidth=2, relief='flat')   # Sets up a frame to hold widgets.
        self.framePlayGame = tk.Frame(master, width=360, height=200, padx=4, pady=4, borderwidth=2, relief='flat')   # Sets up a frame to hold widgets.
        self.frameSetupGame.grid_propagate(0)      # .grid_propagate(0) ensures that a frame stays at its set dimensions when displayed.
        self.framePlayGame.grid_propagate(0)
        self.frameSetupGame.grid(column=0, row=0)  # .grid(...) displays a widget on screen at a column and row position within a user-defined grid.
        self.framePlayGame.grid(column=0, row=1)

        self.frameSetup(master)  # Calls the function 'frameSetup' (passing a reference to 'master', the main program window).

    def frameSetup(self, master):  # This section sets up the contents (labels, buttons, etc.) of the two frames defined in __init__.

        # This section defines the widgets for the first (SetupGame) frame.
        self.labelNumFrom = tk.Label(self.frameSetupGame, font=self.fontMain, text='Number to guess is from : ')
        self.labelNumTo = tk.Label(self.frameSetupGame, font=self.fontMain, text=' to : ')
        self.numhidefault = tk.StringVar()  # This defines a variable to use in Spinbox 'spinboxHi' so that a starting value can be set for it.
        self.numhidefault.set('100')        # This sets the starting value for the 'spinboxHi' variable defined above to '100'.
        self.spinboxLo = tk.Spinbox(self.frameSetupGame, font=self.fontMain, from_=1, to=9991, increment=1, width=4)
        self.spinboxHi = tk.Spinbox(self.frameSetupGame, font=self.fontMain, from_=8, to=9999, increment=1, width=4, textvariable=self.numhidefault)
        # In a Spinbox: 'from_' and 'to' set the range of possible values; 'increment' sets the amount of change when the Spinbox arrows are clicked;
        # 'width' is how many characters the Spinbox can show.
        self.buttonPlay = tk.Button(self.frameSetupGame, font=self.fontMain, text='Play', background='yellow', padx=2, pady=2,
            command=lambda:self.startPlay(master, self.spinboxLo.get(), self.spinboxHi.get()))
        # Buttons contain a 'command' which activates a function when the button is pressed;
        # 'lambda:' is necessary to prevent the function from running when the button is first placed in a window.

        # This section places the widgets for the first (SetupGame) frame and displays them on screen.
        self.labelNumFrom.grid(column=0, row=0)
        self.labelNumTo.grid(column=2, row=0)
        self.spinboxLo.grid(column=1, row=0)
        self.spinboxHi.grid(column=3, row=0)
        self.buttonPlay.grid(column=0, row=1, columnspan=4, pady=8)

        # This section defines the widgets for the second (PlayGame) frame.
        self.labelGuessLimits = tk.Label(self.framePlayGame, font=self.fontMain, text='')
        self.stringMaxGuesses = tk.StringVar()
        self.labelMaxGuesses = tk.Label(self.framePlayGame, font=self.fontMain, textvariable=self.stringMaxGuesses)
        self.labelYourGuess = tk.Label(self.framePlayGame, font=self.fontMain, text='Your guess : ')
        self.stringPlayerGuess = tk.StringVar()
        self.entryGuess = tk.Entry(self.framePlayGame, font=self.fontMain, width=5, textvariable=self.stringPlayerGuess)
        self.buttonGuess = tk.Button(self.framePlayGame, font=self.fontMain, text='Make guess', background='green', padx=2, pady=2,
            command=lambda:self.readGuess(self.numtoguess))
        self.labelComment = tk.Label(self.framePlayGame, font=self.fontMain, text='')
        self.labelSpacer = tk.Label(self.framePlayGame, font=self.fontSpace, text=' ')
        self.buttonNew = tk.Button(self.framePlayGame, font=self.fontMain, text='New game', background='cyan', padx=2, pady=2,
            command=lambda:self.newGame(master))
        self.buttonQuit = tk.Button(self.framePlayGame, font=self.fontMain, text='Quit', background='red', padx=2, pady=2,
            command=lambda:self.endGame(master))

        # This section places the widgets for the second (PlayGame) frame and displays them on screen.
        self.labelGuessLimits.grid(column=0, row=0, columnspan=4, pady=4, sticky='w')
        self.labelMaxGuesses.grid(column=0, row=1, columnspan=4, pady=4, sticky='w')
        self.labelYourGuess.grid(column=0, row=2, sticky='w')
        self.entryGuess.grid(column=1, row=2, pady=4, sticky='w')
        self.buttonGuess.grid(column=3, row=2, pady=4, sticky='e')
        self.labelComment.grid(column=0, row=3, columnspan=4, pady=4, sticky='w')
        self.labelSpacer.grid(column=0, row=4, columnspan=4)
        self.buttonNew.grid(column=0, row=5, columnspan=3, padx=4, pady=4)
        self.buttonQuit.grid(column=3, row=5, padx=4, pady=4)
        self.framePlayGame.grid_remove()   # Hides the second (PlayGame) frame.

    def startPlay(self, master, userNumlo, userNumhi):  # This function closes the SetupGame frame and opens the PlayGame frame when 'Play' is pressed.

        try:                        # Checks if player's choice for the lower number of the range to guess is an integer and gives an error message if not.
            numlo = int(userNumlo)
        except ValueError:
            tm.showwarning('Warning', 'The lower number in the range to guess\nmust be an integer.')
            return
        try:                        # Checks if player's choice for the higher number of the range to guess is an integer and gives an error message if not.
            numhi = int(userNumhi)
        except ValueError:
            tm.showwarning('Warning', 'The higher number in the range to guess\nmust be an integer.')
            return
        if numhi < numlo+7:         # Checks if the range of numbers to guess is suitable.
            tm.showwarning('Warning', 'The higher number in the range to guess\nmust be at least 7 greater than the lower number.')
                                    # Opens a text message box to give a warning message.
        else:
            self.frameSetupGame.grid_remove()          # Hides the first (SetupGame) frame.
            self.framePlayGame.grid()                  # Displays the second (PlayGame) frame.
            self.labelGuessLimits.config(text='The number to guess is between ' + str(numlo) + ' and ' + str(numhi) + '.')
            self.numofguesses = int(m.log(numhi + 1 - numlo, 2) + 1)  # Calculates an appropriate number of guesses for the range of numbers chosen.
            self.numtoguess = r.randint(numlo, numhi)                 # Gives a random number within the range of numbers chosen.
            self.guessesTaken = 0                      # Resets the counter for the number of guesses so far taken by the player to 0.
            self.showGuessesLeft()                     # Calls a function to display the number of guesses left in a label.
            self.labelComment.config(text='')          # Clears the label which displays comments based on the player's guess.
            self.entryGuess.config(state=tk.NORMAL)    # Enables the entry box which allows the player to enter a guess.
            self.buttonGuess.config(state=tk.NORMAL)   # Enables the 'Make guess' button.
            self.buttonNew.config(state=tk.DISABLED)   # Disables the 'New game' button.
            self.buttonQuit.config(state=tk.DISABLED)  # Disables the 'Quit' button.
            self.stringPlayerGuess.set('')             # Clears the contents of the entry box in which the player enters a guess.
            self.entryGuess.focus_set()                # Sets the focus of the cursor so that the player can enter a guess directly.

    def readGuess(self, ntg):  # This function checks the player's guess and reacts accordingly.
        readg = 0                                   # Initializes the player's guess as 0 - necessary for 'try' to work correctly.
        try:                                        # Checks player's guess is an integer and gives an error message if not.
            readg = int(self.entryGuess.get())      # Reads the player's guess into an integer variable 'readg'.
        except ValueError:
            self.labelComment.config(text='This isn\'t an integer - try again.')
            return                                  # The function ends if the guess is not an integer.
        self.guessesTaken +=1                       # Adds one to 'guessesTaken', the counter for the number of guesses taken so far by the player.
        self.showGuessesLeft()                      # Calls a function to display a label showing the number of guesses taken so far.
        if self.guessesTaken <= self.numofguesses:  # This 'if' block gives comments on the player's guess as long as there are guesses left.
            if readg < ntg:
                self.labelComment.config(text='Your guess is too low.')
            elif readg > ntg:
                self.labelComment.config(text='Your guess is too high.')
            elif readg == ntg:
                self.labelComment.config(text='Well done - your guess is correct!')
        if (self.guessesTaken == self.numofguesses) and (readg != ntg):  # This 'if' block gives a comment if the player has run out of guesses without getting the correct answer.
            self.labelComment.config(text='Bad luck - the answer is ' + str(ntg) + '.')
        if (self.guessesTaken >= self.numofguesses) or (readg == ntg):   # This 'if' block prevents the player from making more guesses.
            self.entryGuess.config(state=tk.DISABLED)    # Disables the entry box for the player's guesses.
            self.buttonGuess.config(state=tk.DISABLED)   # Disables the 'Make guess' button.
            self.buttonNew.config(state=tk.NORMAL)       # Enables the 'New game' button.
            self.buttonQuit.config(state=tk.NORMAL)      # Enables the 'Quit' button.

    def showGuessesLeft(self):  # This function changes the content of a variable for use in 'labelMaxGuesses'.
        self.stringMaxGuesses.set('You have ' + str(self.numofguesses-self.guessesTaken) + ' out of ' + str(self.numofguesses) + ' guesses left.')

    def newGame(self, master):  # This functions hides the game-play frame and displays the set-up frame.
        self.frameSetupGame.grid()
        self.framePlayGame.grid_remove()

    def endGame(self, master):  # This function ends the game - both 'destroy' and 'quit' are necessary.
        master.destroy()        # Closes the program window.
        master.quit()           # Ends the program.


def main():
    root = tk.Tk()   # Sets up an empty program window.
    app = App(root)  # Runs the application as an instance of Class App in the window set up above (i.e. starts the program).
    root.mainloop()  # Makes the program continue until ended by the user.


if __name__ == '__main__':  # This runs  main()  if this module has not been imported into another module.
    main()
